package frc.robot.interfaces;

import java.util.OptionalDouble;

/**
 * An interface which ensures a class can provide common AprilTag oriented
 * information from various sources in a consistent way.
 */
public interface IAprilTagProvider {
    /**
     * A method to get the AprilTag offset from the center of the image. This is intended to be used with 
     * a PIDController, the PIDController keeps the robot rotated such that the AprilTag stays in the horizontal
     * center of the image. 
     * 
     * TODO Is there some other way of ensuring an AprilTag is in the center of the image frame without knowing the horizontal resolution?
     * TODO Certain providers don't give resolution information natively. Which makes me think there's some other way to do this.
     * 
     * @param id The ID of the AprilTag to give an offset from
     * @param cameraHalfXRes Half of the horizontal resolution of the camera sensor
     * @return A double with the number of pixels offset from center the tag is, or OptionalDouble.empty() if the tag is not present in the camera's view
     */
    public OptionalDouble getTagOffsetFromCameraCenterByID(int id, double cameraHalfXRes);

    /**
     * A method to get the distance from <i>the camera</i> to the AprilTag specified
     * 
     * @param id The ID of the AprilTag to give a distance to
     * @param targetHeightMeters The height of the AprilTag off the ground, in meters
     * @return The distance, in meters, to the target, or OptionalDouble.empty() if the tag is not present in the camera's view
     */
    public OptionalDouble getTagDistanceFromCameraByID(int id, double targetHeightMeters);

    /**
     * A method to get the pitch of a particular AprilTag
     * 
     * @param id The ID of the AprilTag to get the pitch of
     * @return The pitch, in radians, of the target, or OptionalDouble.empty() if the tag is not present in the camera's view
     */
    public OptionalDouble getTagPitchByID(int id);

    /**
     * A method to get the yaw of a particular AprilTag
     * 
     * @param id The ID of the AprilTag to get the yaw of
     * @return The yaw, in radians, of the target, or OptionalDouble.empty() if the tag is not present in the camera's view
     */
    public OptionalDouble getTagYawByID(int id);
}
